-- ASU_ROP2_16.vhd
-- 20081116 : 16-bit only subset, without pipeline,
--  no optimisation (but still functionally equivalent to the
--  Actel A3P optimised code).
-- 20081228 : restored the ActelProASIC3 version.
-- max speed for 7 passes on A3P250 standard speed :
--   ActelProASIC3 : 120MHz (98 according to Synplify)
--   behaviour : 96MHz (no add_EN)
-- 20090108 : add_ENr moved to input
-- 20090603 : changed std_logic to std_ulogic
-- 20090813 : added signed comparisons,
--    proof-checked behaviour against ActelProASIC3,
--    changed port names : DataA => si4, DataB => snd,
--    will ROP2_combine be obsoleted ?
-- 20090824

-- This unit performs the addition, substraction and all
-- logic operations ("ROP2") for the YASEP in the special case
-- of a 16-bit wide datapath.
-- * ASU is also used by UMIN, UMAX (the carry output can disable
--     the writeback of data), comparisons (signed and unsigned) and
--     as a last stage of multiplies.
-- * ROP2 is also used by the conditional jumps (combined output) (??)
--
-- Warning : ROP2 data output is available before ASU output.
-- * ROP2 output at L5, there's 1 gate level left for MUX2 before latches
-- * ASU outputs is L8, 2 levels after the latch.

library IEEE;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ASUROP2_16 is
  generic (
    PIPELINED  : integer := 0; -- 1=pipelined
    YASEP_SIZE : integer := 16 -- specific to this implementation
  );
  port(
    SND, SI4 : in std_ulogic_vector(YASEP_SIZE-1 downto 0);
    AuxLo, AuxHi : in std_ulogic_vector(11 downto 0);
  -- Add/Sub
    Addsub, clk, MulMux, add_EN, add_ENr, compare_signed : in std_ulogic;
    Sum : out std_ulogic_vector(YASEP_SIZE downto 0);
  -- ROP2 :
    ROP2_S1, ROP2_S2, ROP2_X2 : in std_ulogic; -- ROP2 functions
    ROP2_OUT : out std_ulogic_vector(YASEP_SIZE-1 downto 0);
    ROP2_combine : out std_ulogic);
end ASUROP2_16;

--         ROP2_S1  ROP2_S2  ROP2_X2  Addsub  compare_signed
-- ADD        X        X        X       0           0
-- SUB        X        X        X       1           0
-- compare    X        X        X       1 (== SUB, only carry-out used)
--    signed  X        X        X       1           1
--  unsigned  X        X        X       1           0
-- AND        1        0        0       0           0
-- ANDN       1        0        0       1           0
-- NAND       1        0        1       0           0
-- OR         X        1        0       0           0
-- ORN        X        1        0       1           0
-- NOR        X        1        1       0           0
-- XOR        0        0        0       0           0
-- XORN       0        0        0       1           0
-- equality   0        0        0       0           0 ( == reduceOR(XOR) )

-----------------------------------------------------------------

architecture behaviour of ASUROP2_16 is
begin
  process (SI4, SND, Addsub, clk, add_EN, add_ENr,
    AuxLo, AuxHi, MulMux, ROP2_S1, ROP2_S2, ROP2_X2, compare_signed)

    variable sumAux : unsigned(Sum'left+1 downto 0);
    variable ActualA, ActualB, ActualAr, ActualBr,
        ROP2_RS1, ROP2_RS2 : std_ulogic_vector(SND'range);
    variable Addsubr, ROP2_S1r, ROP2_S2r, ROP2_X2r : std_ulogic;

  begin

    assert (YASEP_SIZE=16)
      report "YASEP_SIZE must be 16"
      severity failure;

    if (MulMux = '0') then
      ActualB((YASEP_SIZE-2) downto 0) := SND((YASEP_SIZE-2) downto 0)
                                          xor((YASEP_SIZE-2) downto 0 =>Addsub);
      ActualB(YASEP_SIZE-1) := SND(YASEP_SIZE-1)
                               xor (Addsub and (not compare_signed));
      -- initial XOR (usually merged/combined with the first stage of ASU)
      -- this part also deals with the XOR of the MSB of SND that deals with signed comparisons.

      ActualA((YASEP_SIZE-2) downto 0) := SI4((YASEP_SIZE-2) downto 0);
      -- XOR the MSB here too for the signed comparisons :
      ActualA(YASEP_SIZE-1) := SI4(YASEP_SIZE-1) xor compare_signed;
    else
      ActualA := "0000" & AuxLo;
      ActualB := AuxHi & "0000";
    end if;

-- pipeline barrier #1
    if (PIPELINED=0) or (clk'event and clk='1' and add_en='1') then
      ActualAr := ActualA;
      ActualBr := ActualB;
      Addsubr  := Addsub;
      ROP2_S1r := ROP2_S1;
      ROP2_S2r := ROP2_S2;
      ROP2_X2r := ROP2_X2;
    end if;

    -- add/sub :
    sumAux := unsigned('0' & ActualAr & '1') + unsigned('0' & ActualBr & Addsubr);
    --    if (Addsub = '0') then + else -

    -- ROP2 :
    if (ROP2_S1='1') then -- First MUX
      ROP2_RS1 := ActualAr and ActualBr;
    else
      ROP2_RS1 := ActualAr xor ActualBr;
    end if;

    if (ROP2_S2='1') then -- second mux
      ROP2_RS2 := ActualAr  or ActualBr;
    else
      ROP2_RS2 := ROP2_RS1;
    end if;

    ROP2_OUT <= ROP2_RS2 xor (ROP2_OUT'range => ROP2_X2r); -- last XOR

    -- OR of all the XORs (reduction, used for comparisons)
    -- The source is not "SI4 xor DataB2" but ROP2_RS1
    -- so that the fanout of the XOR is not excessive
    -- (it is used by add/sub too !).
    -- This may be pointless with a LUT-4 architecture, though.
    if ROP2_RS1=(ROP2_RS1'range=>'0') then
      ROP2_combine <= '0';
    else
      ROP2_combine <= '1';
    end if;

-- pipeline barrer #2
    if (PIPELINED=0) or (clk'event and clk='1' and add_enr='1') then
      -- trim the LSB (carry-in)
      Sum(Sum'left downto 0) <= std_ulogic_vector(sumAux(sumAux'left downto 1));
    end if;

  end process;
end behaviour;


--------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------
-- Gate-level version :
--------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------


architecture ActelProASIC3 of ASUROP2_16 is
begin

  -- all the unit's inputs again :-/
  process (SI4, SND, Addsub, clk, add_EN, add_ENr,
    AuxLo, AuxHi, MulMux, ROP2_S1, ROP2_S2, ROP2_X2, compare_signed)

  variable DataAReg, DataBReg, ActualA, ActualB,
         uDataB_s1in, Gene, Prop, Comp, -- preliminary inversions
         ROP2_RS1, ROP2_RS2 : -- ROP2 temporary results
           std_ulogic_vector(YASEP_SIZE-1 downto 0);
  variable DataB0, DataB0_r, Sum32_or, N391r, Sum0r, Sum0rr, Sum1rr, Sum2rr,
          Prop1pre, Prop1preR, Gene1pre, Gene1preR,
          N555_dup, N621_dup, N619_dup, N594_dup, N662_dup, -- optimisation CDP
          ROP2_c,
-- latches level 2 :
--L2
          Comp5r, Comp7r, Comp8r, Prop8r, Gene8r, Comp9r, Comp10r, Comp11r, Comp12r, Gene12r,
          Prop12r, Comp13r, Comp14r, Comp15r,
--L3
          N109_1r, N105_1r, N524r, N516r,
--L4
          N611r, I300Y1r,
--L5
          I271un1Y0r, I270Y0a01r, I358YN4ir, N4ir, N562r, N630r, N632r, N634r,
          N638r, N642r, N646r, N647r, N650r, N657r, N658r, N662r, N663r,
          N678r, N662_dupr, I208un1Yr, I268Y2r,
-- L6
          Sum3r, Sum4r, I210Y0a3r, Sum6r, N926r, N920r, N917r, N914r,
          N908r, I250un1Yir, N731r, N732r, I244un1Yr, I281un1Y0r, I280un1Yr, I280Y0r,
          I279Y0r, I240un1Yr, I277un1Yr, I277Y0r, I276un1Y0r, I276Y0r, I275Y0r, I273Y1r,
          I272un1Y0r, I272Y1r, I271Y1r, I270Y2r, I269Y2r, I268un1Y0r, I300un1Y0r, I300Y3r, -- 85 latches

  I94un1Y, N391, -- 1
  N621, N554, N555, N555_Ain, N555_Aout, -- 2
  N619, -- 3
  N685, N617, N618, N550, N551, -- 4
  I210Y0a3, -- 5
  N681, N613, N614, N546, N547, -- 6
  I208un1Y, N611, N612, N_42, N548i, I146Y0, -- 7
  N926, N678, N677, N610, N609, N543, N542, -- 8
  I256Y0a3, -- 9
  N920, N674, N673, N605, N606, N538, N539, -- 10
  N917, N672, N671, N604, I139Y0, N_40, N603, N540i, I138Y0, -- 11
  N914, N602, N601, N669, N670, N535, N534, -- 12
  N911i, -- 13
  N908, N665, N666, N598, N531, N597, N530, -- 14
  N663, I250un1Yi, N595, N596, N38i0, N532, I130Y0, N664, -- 15
  N526, N527, N661, N662, N594, N593, N731, N732 : std_ulogic;  -- 16

begin

    assert (YASEP_SIZE=16)
      report "YASEP_SIZE must be 16"
      severity failure;

-- L0 : SND, SI4, Addsub, compare_signed

------------------------------------------------------------
------------------------------------------------------------
-- L1 + MUX2
  if (MulMux = '0') then
-- These are the normal data for addition
    ActualA((YASEP_SIZE-2) downto 0) := SI4((YASEP_SIZE-2) downto 0);
    ActualA(YASEP_SIZE-1) := SI4(YASEP_SIZE-1) xor compare_signed;

-- inversions prliminaires L1(0,0)
    ActualB((YASEP_SIZE-2) downto 0) := SND((YASEP_SIZE-2) downto 0) xor ((YASEP_SIZE-2) downto 0 => Addsub);
    ActualB(YASEP_SIZE-1) := SND(YASEP_SIZE-1) xor (Addsub and (not compare_signed));

    DataB0_r := SND(0);
--  I333_Y fo1 L1(0,0)
    Sum0r := SI4(0) xor SND(0);
-- dcritisation d'un chemin :
    Prop1preR := (SND(1) xor Addsub) or  SI4(1);
    Gene1preR := (SND(1) xor Addsub) and SI4(1);
-- Sum1
  -- I0_G0N fo2 L1(0,0)
    N391r := SI4(0) and Addsub;
-- Sum2
    N555_Ain := SI4(0) or Addsub;
  else
-- This is the data that comes from the multiplier
-- Hints : AuxHi(0 to 3)='0' and addsub='0'
    ActualA := "0000" & AuxLo;  -- SI4
    ActualB := AuxHi & "0000";  -- SND
    DataB0_r := '0';       -- :=              (AuxHi(0)=0)
    Sum0r    := AuxLo(0);  -- := AuxLo(0) xor (AuxHi(0)=0);
    N391r   := '0';        -- := AuxLo(0) and (Addsub=0);
    N555_Ain := AuxLo(0);  -- := AuxLo(0) or  (Addsub=0);
    Prop1preR := AuxLo(1); -- := AuxLo(1) or  (Addsub=0);
    Gene1preR := '0';      -- := AuxLo(1) and (Addsub=0);
  end if;

------------------------------------------------------------
------------------------------------------------------------
-- pipeline gate #1:
  if (PIPELINED=0) or (clk'event and clk='1' and add_en='1') then
    DataAReg  := ActualA; -- Harmless warning : "Pruning bit <xx> of dataareg(15 downto 0) - not in use ... "
    DataBReg  := ActualB;
    DataB0    := DataB0_r;
    N555_Aout := N555_Ain;
    N391      := N391r;
    Sum0rr    := Sum0r;
    Prop1pre  := Prop1preR;
    Gene1pre  := Gene1preR;
  end if;

------------------------------------------------------------
------------------------------------------------------------
-- L2

-- Generate, Propagate, Compute : L2(1,0)
  Gene := DataBReg and DataAReg;
  Prop := DataBReg  or DataAReg;
  Comp := DataBReg xor DataAReg;

-- Sum1
  -- fo1 L2(0,0,1)
    I94un1Y := (DataAReg(0) or not(DataB0)) and DataBReg(0);

-- Sum2
  -- fo2 L2(1,1,1)
    N554 := (N391 and Prop1pre) or Gene1pre;
  -- I93_Y fo3 L2(1,1)
    N555 := N555_Aout and Prop1pre;
  -- I93_Y fo1 L2(1,1)
    N555_dup := N555_Aout and Prop1pre and DataBReg(0);

------------------------------------------------------------
------------------------------------------------------------
-- L3

-- Sum1
  -- L3(2,1,2)
    Sum1rr := (I94un1Y or N391) xor Comp(1);

-- Sum2
  -- fo3 L3(2,1,2) 
    N621 := (N555 and DataBReg(0)) or N554;
  -- fo2 L3(2,2)
    N621_dup := N555_dup or N554;

-- Sum4
  -- fo2 L3(1,0,2)
    N550 := (DataBReg(3) and DataAReg(3))
         or (DataBReg(3) and Gene(2))
         or (DataAReg(3) and Gene(2));
  -- fo3 L3(2,2)
    N551 := Prop(3) and Prop(2);

-- Sum6
  -- fo2 L3(1,0,2)
    N546 := (DataBReg(5) and DataAReg(5))
         or (DataBReg(5) and Gene(4))
         or (DataAReg(5) and Gene(4));
  -- I85_Y fo3 L3(2,2)
    N547 := Prop(5) and Prop(4);

-- Sum7
  -- fo2 L3(2,2)
    N_42 := Prop(5) and Prop(6);
  -- fo1 L3(2,2,2)
    N548i := (Gene(3) and Prop(4)) or Gene(4);
  -- fo1 L3(1,0,2)
    I146Y0 := (DataBReg(6) and DataAReg(6))
           or (DataBReg(6) and Gene(5))
           or (DataAReg(6) and Gene(5));

-- Sum8
  -- fo2 L3(1,0,2)
    N542 := (DataBReg(7) and DataAReg(7))
         or (DataBReg(7) and Gene(6))
         or (DataAReg(7) and Gene(6));
  -- I81_Y fo3 L3(2,2)
    N543 := Prop(7) and Prop(6);

-- Sum10
  -- fo2 L3(1,0,2)
    N538 := (DataBReg(9) and DataAReg(9))
         or (DataBReg(9) and Gene(8))
         or (DataAReg(9) and Gene(8));
  -- fo3 L3(2,2)
    N539 := Prop(9) and Prop(8);

-- Sum11
  -- fo1 L3(2,2,2)
    I138Y0 := (Gene(9) and Prop(10)) or Gene(10);
  -- fo1 L3(2,2)
    I139Y0 := Prop(7) and Prop(8);
  -- fo2 L3(2,2)
    N_40 := Prop(9) and Prop(10);
  -- fo1 L3(2,2,2)
    N540i := (Gene(7) and Prop(8)) or Gene(8);

-- Sum12
  -- fo2 L3(2,0,1)
    N534 := (DataBReg(11) and DataAReg(11))
         or (DataBReg(11) and Gene(10))
         or (DataAReg(11) and Gene(10));
  -- fo3 L3(2,2)
    N535 := Prop(11) and Prop(10);

-- Sum14
  -- fo2 L3(2,1,0)
    N530 := (DataBReg(13) and DataAReg(13))
         or (DataBReg(13) and Gene(12))
         or (DataAReg(13) and Gene(12));
  -- fo3 L3(2,2)
    N531 := Prop(13) and Prop(12);

-- Sum15
  -- fo1 L3(2,2,2)
    N532 := (Gene(11) and Prop(12)) or Gene(12);
  -- fo2 L3(2,2)
    N38i0 := Prop(14) and Prop(13);
  -- fo1 L3(2,1,0)
    I130Y0 := (DataBReg(14) and DataAReg(14))
           or (DataBReg(14) and Gene(13))
           or (DataAReg(14) and Gene(13));

-- Sum16
  -- fo2 L3(1,0,2)
    N526 := (DataBReg(15) and DataAReg(15))
         or (DataBReg(15) and Gene(14))
         or (DataAReg(15) and Gene(14));
  -- fo3  L3(2,2)
    N527 := Prop(15) and Prop(14);

------------------------------------------------------------
------------------------------------------------------------
-- L4

-- Sum3
  -- fo4 L4(3,2,2)
    N619 := (N621_dup and Prop(2)) or Gene(2);
  -- fo4 L4(3,2,2)
    N619_dup := N621_dup and Prop(2);

-- Sum4
  -- fo2 L4(3,3,3)
    N617 := (N551 and N554) or N550;
  -- fo2 L4(3,3)
    N618 := N555 and N551;

-- Sum6
  -- fo2 L4(3,3,3)
    N613 := (N547 and N550) or N546;
  -- fo2 L4(3,3)
    N614 := N547 and N551;

-- Sum7
  -- fo4 L4(3,3,3)
    N611 := (N548i and N_42) or I146Y0;
  -- fo2 L4(2,2,3)
    N612 := Prop(4) and Prop(3) and N_42;

-- Sum8
  -- fo2 L4(3,3,3)
    N609 := (N543 and N546) or N542;
  -- fo3 L4(3,3)
    N610 := N547 and N543;

-- Sum10
  -- fo2 L4(3,3,3)
    N605 := (N539 and N542) or N538;
  -- fo3 L4(3,3)
    N606 := N543 and N539;

-- Sum11
  -- fo2 L4(3,3,3)
    N603 := (N540i and N_40) or I138Y0;
  -- fo3 L4(3,3)
    N604 := I139Y0 and N_40;

-- Sum12
  -- fo2 L4(3,3,3
    N601 := (N535 and N538) or N534;
  -- fo4 L4(3,3)
    N602 := N539 and N535;

-- Sum14
  -- fo2 L4(3,3,3)
    N597 := (N531 and N534) or N530;
  -- fo3 L4(3,3)
    N598 := N535 and N531;

-- Sum15
  -- fo2 L4(3,3,3)
    N595 := (N532 and N38i0) or I130Y0;
  -- fo3 L4(2,2,3)
    N596 := Prop(11) and Prop(12) and N38i0;
    
-- Sum16
  -- fo2 L4(3,3,3)
    N593 := (N527 and N530) or N526;
  -- fo5 L4(3,3)
    N594 := N531 and N527;
    N594_dup := N531 and N527;

------------------------------------------------------------
------------------------------------------------------------
-- L5

-- Sum2
  -- L5(2,4)
    Sum2rr := Comp(2) xor N621;

-- Sum4
  -- fo4 L5(1,4,4)
    N685 := (DataBReg(0) and N618) or N617;

-- Sum6
  -- fo3 L5(4,4,4)
    N681 := (N614 and N621) or N613;

-- Sum7
  -- fo3 L5(4,4)
    I208un1Y := (N619_dup or Gene(2)) and N612;

-- Sum8
  -- fo2 L5(4,4,4)
    N677 := (N610 and N617) or N609;
  -- fo2 L5(4,4)
    N678 := N618 and N610;

-- Sum10
  -- fo2 L5(4,4,4)
    N673 := (N606 and N613) or N605;
  -- fo2 L5(4,4)
    N674 := N614 and N606;

-- Sum11
  -- fo2 L5(4,4,4)
    N671 := (N604 and N611) or N603;
  -- fo2 L5(4,4)
    N672 := N612 and N604;

-- Sum12 ??

-- Sum13
  -- fo2 L5(4,4,4)
    N669 := (N602 and N609) or N601;
  -- fo2 L5(4,4)
    N670 := N610 and N602;
    
-- Sum14
  -- fo2 L5(4,4,4)
    N665 := (N598 and N605) or N597;
  -- fo2 L5(4,4)
    N666 := N606 and N598;

-- Sum15
  -- fo4 L5(4,4,4)
    N663 := (N596 and N603) or N595;
  -- fo2 L5(4,4)
    N664 := N604 and N596;

-- Sum16
  -- fo2 L5(4,4,4)
    N661 := (N594 and N601) or N593;
  -- fo3 L5(4,4)
    N662 := N602 and N594_dup;
    N662_dup := (N539 and N535) and N594_dup;

------------------------------------------------------------
------------------------------------------------------------
-- L6

-- Sum3
  -- L6(2,5)
    Sum3r := Comp(3) xor N619;

-- Sum4
  -- L6(2,5)
    Sum4r := Comp(4) xor N685;

-- Sum5
  -- fo1 L6(5,2,2)
    I210Y0a3 := (N685 and Prop(4)) or Gene(4);

-- Sum6
  -- L6(2,5)
    Sum6r := Comp(6) xor N681;

-- Sum8
  -- fo3 L6(5,5)
    N926 := (DataBReg(0) and N678) or N677;

-- Sum10
  -- fo2 L6(5,4,5)
    N920 := (N674 and N621) or N673;

-- Sum11
  -- fo2 L6(5,5,5)
    N917 := (N619 and N672) or N671;

-- Sum12
  -- fo3 L6(5,5,5)
    N914 := (N670 and N685) or N669;
    
-- Sum14
  -- fo2 L6(5,5,5)
    N908 := (N666 and N681) or N665;

-- Sum15
  -- fo3 L6(5,4,5)
    I250un1Yi := (I208un1Y or N611) and N664;

-- Sum16
  -- fo2 L6(5,5,5)
    N731 := (N662 and N677) or N661;
  -- fo2 L6(5,5,5)
    N732 := N678 and N662 and DataBReg(0);


------------------------------------------------------------
------------------------------------------------------------
-- ROP2

 -- L3 : Mux1
  if (ROP2_S1='1') then
    ROP2_RS1 := Gene; -- and
  else
    ROP2_RS1 := Comp; -- xor
  end if;

 -- L4 :
  -- Mux2
  if (ROP2_S2='1') then
    ROP2_RS2 := Prop; -- or
  else
    ROP2_RS2 := ROP2_RS1;
  end if;

-- the OR-reduction is pushed AFTER the L3 MUX2
-- so the fanout on Comp is reduced.
-- The reduction tree takes 3 3-input logic levels
-- so it goes from L4 to L6.
  if ROP2_RS1=(ROP2_RS1'range=>'0') then
    ROP2_c := '0';
  else
    ROP2_c := '1';
  end if;

  -- L5 : XOR2 (invert the result if ROP2_X2 is set)
  ROP2_OUT <= ROP2_RS2 xor (ROP2_OUT'range => ROP2_X2);

  -- L6 : external MUX2 of ROP2_OUT and end of the OR-tree

------------------------------------------------------------
------------------------------------------------------------
-- pipeline gate #2 :


  if (PIPELINED=0) or (clk'event and clk='1' and add_ENr='1') then
    Sum(0)   <= Sum0rr;
    Sum(1)   <= Sum1rr;
    Sum(2)   <= Sum2rr;
  --L2
    Comp5r   := Comp(5);
    Comp7r   := Comp(7);
    Comp8r   := Comp(8);
    Prop8r   := Prop(8);
    Gene8r   := Gene(8);
    Comp9r   := Comp(9);
    Comp10r  := Comp(10);
    Comp11r  := Comp(11);
    Comp12r  := Comp(12);
    Gene12r  := Gene(12);
    Prop12r  := Prop(12);
    Comp13r  := Comp(13);
    Comp14r  := Comp(14);
    Comp15r  := Comp(15);

    --L4
    N611r    := N611;

    --L5
    N663r      := N663;
    I208un1Yr  := I208un1Y;

    -- L6
    Sum(3)     <= Sum3r;
    Sum(4)     <= Sum4r;
    I210Y0a3r  := I210Y0a3;
    Sum(6)     <= Sum6r;
    N926r      := N926;
    N920r      := N920;
    N917r      := N917;
    N914r      := N914;
    N908r      := N908;
    N731r      := N731;
    N732r      := N732;
    I250un1Yir := I250un1Yi;

    ROP2_combine <= ROP2_c;
	end if;

--------------------------------------------------------------
------------------------------------------------------------
-- L7

-- Sum5
  -- L7(2,6)
    Sum(5) <= Comp5r xor I210Y0a3r;

-- Sum7
  -- L7(6,4,2)
    Sum(7) <= (I208un1Yr or N611r) xor Comp7r;

-- Sum8
  -- L7(2,6)
    Sum(8) <= Comp8r xor N926r;

-- Sum9
  -- fo1 L7(6,2,2)
    I256Y0a3 := (N926r and Prop8r) or Gene8r;

-- Sum10
  -- L7(2,6)
    Sum(10) <= Comp10r xor N920r;

-- Sum11
  -- L7(2,6)
    Sum(11) <= Comp11r xor N917r;

-- Sum12
  -- L7(2,6)
    Sum(12) <= Comp12r xor N914r;

-- Sum13
  -- fo1 L7(6,2,2)
    N911i := (Prop12r and N914r) or Gene12r;

-- Sum14
  -- L7(6,2)
    Sum(14) <= Comp14r xor N908r;

-- Sum16
  -- L7(6,6,2)
    Sum(16) <= (N731r or N732r);

------------------------------------------------------------
------------------------------------------------------------
-- L8

-- Sum9
  -- I342_Y  L8(7,2)
    Sum(9) <= I256Y0a3 xor Comp9r;

-- Sum13
  -- I346_Y L8(2,7)
    Sum(13) <= Comp13r xor N911i;

-- Sum15
  -- I348_Y L8(5,7,2)
    Sum(15) <= (N663r or I250un1Yir) xor Comp15r;

------------------------------------------------------------

end process;
end ActelProASIC3;
