-- SHL_16.vhd
-- Version 20090101 by whygee@f-cpu.org
-- version 20090108 : moved SHL_enr to input
-- version 20090112 : removed SHL_enr
-- version 20090813 : new port names

-- This code implements the SHuffLe units of YASEP-16
-- P&R indicate a 140MHz performance (pipelined).
-- The preliminary stage spans 3 logic layers.

--      Arithmetic Rotation  Direction
-- SHR       0         0         0
-- SHL       0         0         1
-- ROR       0         1         0
-- ROL       0         1         1
-- SAR       1         0         0
-- note: Arithmetic excludes Direction and Rotation (or it won't work)

library IEEE;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity SHL_16 is
  generic (
  	PIPELINED  : integer := 1; -- 1=pipelined
    YASEP_SIZE : integer := 16 -- must be 16
  );
  port(
    SND : in std_ulogic_vector(YASEP_SIZE-1 downto 0);
    SI4 : in std_ulogic_vector(3 downto 0);
    clk, SHL_en, Rotation, Direction, Arithmetic : in std_ulogic;

    ResultSHL : out std_ulogic_vector(YASEP_SIZE-1 downto 0)
  );
end SHL_16;

-----------------------------------------------------------------

architecture behaviour of SHL_16 is
begin

  assert (YASEP_SIZE=16)
    report "YASEP_SIZE must be 16"
    severity failure;

  process (SND, SI4, clk, SHL_en,
           Rotation, Direction, Arithmetic)

    variable SHAr, SHA2, SHA2r, SHA3, SHA3r : std_ulogic_vector(3 downto 0);
    variable shl1, shl1Reg, shl2, shl4, shl8,
        mask, maskAND, maskOR, res : std_ulogic_vector(ResultSHL'range);
    variable RotReg, DirReg, AriReg, signext, signextr,
        DoSHLr, DoSHL : std_ulogic;


    begin

      -- ROR(1)
      -- Note that SI4(0)=SHA2(0) because negation conserves parity !
      if (SI4(0)='1') then
        shl1 := SND(0) & SND(15 downto 1);
      else
        shl1 := SND;
      end if;

      -- Correction of the SHA for the left shift
      SHA3 := SI4 xor (SHA3'range=>Direction);

      -- correction for the barrel shifter :
      -- reverse the actual shift amount according to the direction
--      SHA2 := std_ulogic_vector( unsigned(SI4 xor (SHA3'range=>Direction))
--                 + unsigned(std_ulogic_vector'("000") & Direction));
-- old version : works better by not factoring the XOR, saves 1 logic layer
      if (Direction='1') then
        SHA2 := std_ulogic_vector( unsigned(not(SI4)) +1 );
      else
        SHA2 := SI4;
      end if;

      -- value of the sign bit that will be propagated
      signext := Arithmetic and SND(15);

      -- whether the mask mush be XORed
      DoSHL := not(Rotation) and Direction;

      -- First pipeline gate
      if (PIPELINED=0) or (clk'event and clk='1' and SHL_en='1') then
        shl1Reg := shl1;
        signextr := signext;
        RotReg  := Rotation;
        DirReg  := Direction;
        AriReg  := Arithmetic;
        SHAr    := SI4;
        SHA2r   := SHA2; -- Harmless warning "Pruning bit <0> of sha2r_1(3 downto 0) - not in use ..."
        SHA3r   := SHA3;
        DoSHLr  := DoSHL;
      end if;


-- The rest of the barrel shifter
      -- ROR(2)
      if (SHA2r(1)='1') then
        shl2 := shl1Reg(1 downto 0) & shl1Reg(15 downto 2);
      else
        shl2 := shl1Reg;
      end if;

      -- ROR(4)
      if (SHA2r(2)='1') then
        shl4 := shl2(3 downto 0) & shl2(15 downto 4);
      else
        shl4 := shl2;
      end if;

      -- ROR(8)
      if (SHA2r(3)='1') then
        shl8 := shl4(7 downto 0) & shl4(15 downto 8);
      else
        shl8 := shl4;
      end if;

-- the masking and decoders
-- (a more compact/iterative code would be cool)
      case SHA3r is
        when "0000" => mask:= "1111111111111111";
        when "0001" => mask:= "0111111111111111";
        when "0010" => mask:= "0011111111111111";
        when "0011" => mask:= "0001111111111111";
        when "0100" => mask:= "0000111111111111";
        when "0101" => mask:= "0000011111111111";
        when "0110" => mask:= "0000001111111111";
        when "0111" => mask:= "0000000111111111";
        when "1000" => mask:= "0000000011111111";
        when "1001" => mask:= "0000000001111111";
        when "1010" => mask:= "0000000000111111";
        when "1011" => mask:= "0000000000011111";
        when "1100" => mask:= "0000000000001111";
        when "1101" => mask:= "0000000000000111";
        when "1110" => mask:= "0000000000000011";
        when "1111" => mask:= "0000000000000001";
        when others => mask:= (mask'range => '1');
      end case;

      maskAND := (mask xor (mask'range => DoSHLr))  -- SHL
         or (mask'range => RotReg); -- ROR/ROL
      maskOR := not(mask) and (mask'range => signextr); -- SAR

-- the last level : OA1
      ResultSHL <= (shl8 and maskAND) or maskOR;

    end process;
end behaviour;
